#include <allegro.h>
#include "input.h"
#include "graphics.h"
#include "sprite.h"
#include "killmario.h"
#include "timer.h"
#include "minigame.h"

void KILLMARIO::DisplayTimer()
{
     alfont_textprintf_ex(back_buffer, standardFont, 10, 10, makecol(255, 255, 255), makecol(0, 0, 0), "TIME: %d", gameTimer.GetValueFloor());
}

void KILLMARIO::DrawFireball()
{
     int counter = 0;
     bool alreadyStartedOne = false;
     while(counter < 3)
     {
         if(mario.alive && fire[counter].onscreen && fire[counter].fired)
         {
             //Check for collision.
             if(Collision(&fireball,fire[counter].x.GetValueFloor(), 300 - 50, &marioAlive, 140, mariosPlatform.y.GetValueFloor() - marioAlive.GetHeight()))
             {
              mariosPlatform.y.RemoveFromProcessingList();                                                       
              mario.alive = false;
              mario.y = mariosPlatform.y.GetValueFloor() - marioAlive.GetHeight();
              mario.speed.SetValue(-300 - difficultySpeed);
              mario.speed.SetUPS(600 + (6 * difficultySpeed));
              mario.speed.AddToProcessingList();
              sizzleSound.PlaySound();
             }
         }          
         if(fire[counter].onscreen && fire[counter].onscreen && fire[counter].x.GetValueFloor() < (-1 * fireball.GetWidth()))
         {
             fire[counter].onscreen = false;
             if(mario.alive)
             fire[counter].missed = true;
             fire[counter].x.RemoveFromProcessingList();
         }                   
         if(!alreadyStartedOne && ButtonADown() && !fire[counter].fired)
         {
             alreadyStartedOne = true;                  
             fire[counter].onscreen = true;
             fire[counter].fired = true;
             fire[counter].x.SetValue(480);
             fire[counter].x.SetUPS(-600 - difficultySpeed);
             fire[counter].x.AddToProcessingList();
             fireSound.PlaySound();
         }
         if(fire[counter].onscreen)
         {
             fireball.DisplayMaskedXY(fire[counter].x.GetValueFloor(), 300 - 50);
         }
         counter++;
     }
}

void KILLMARIO::DrawMario()
{
     if(mario.alive)
     {
         marioAlive.DisplayMaskedXY(140, mariosPlatform.y.GetValueFloor() - marioAlive.GetHeight());
     }
     else
     {
         marioDead.DisplayMaskedXY(140, (int)mario.y);
         mario.y = mario.y + (mario.speed.GetValue() * FrameTime());
     }
}

void KILLMARIO::DrawPlatform()
{
     if(mariosPlatform.y.GetValueFloor() >= mariosPlatform.lowerValue)
     {
         mariosPlatform.y.SetUPS(-1 * (mariosPlatform.speed + difficultySpeed));
     }
     if(mariosPlatform.y.GetValueFloor() <= mariosPlatform.topValue)
     {
         mariosPlatform.y.SetUPS(mariosPlatform.speed + difficultySpeed);
     }
     platform.DisplayMaskedXY(100, mariosPlatform.y.GetValueFloor());
}

void KILLMARIO::DrawBowser(void)
{
     if(ButtonAPressed())
     {
         bowserFiring.DisplayMaskedXY(500, 300 - bowserFiring.GetHeight());
     }
     else
     {
         bowserStill.DisplayMaskedXY(500, 300 - bowserStill.GetHeight());
     }
}

void KILLMARIO::DrawFireballsLeft(void)
{
     int counter = 0;
     while(counter < 3)
     {
         if(!fire[counter].fired)
         {
             fireballsLeft.DisplayMaskedXY(500 + 40 * counter, 10);
         }
         counter++;
     }
}

void KILLMARIO::DrawBackgroud(void)
{
     int xcount;
     ClearScreen(0, 0, 0);
     
     //Draw lava at bottom of screen.
     for(xcount = 0; xcount < 640; xcount+=lava.GetWidth())
     {
         lava.DisplayXY(xcount, 480 - lava.GetHeight());
     }
     
     for(xcount = 400; xcount < 640; xcount+=bridge.GetWidth())
     {
         bridge.DisplayXY(xcount, 300);
     }
}

void KILLMARIO::LoadArt(void)
{
     MINIGAME::LoadArt();
    marioAlive.LoadImage("mario1.bmp");
    marioDead.LoadImage("mario2.bmp");
    platform.LoadImage("marioplat.bmp");   
    bowserStill.LoadImage("bowsera.bmp");
    bowserFiring.LoadImage("bowserb.bmp");
    fireball.LoadImage("fireball.bmp");
    lava.LoadImage("lava.bmp");
    bridge.LoadImage("bridge.bmp");
    fireballsLeft.LoadImage("fire.bmp");
    fireSound.LoadSound("breath.mp3");
    sizzleSound.LoadSound("sizzle.mp3");
}

void KILLMARIO::ReleaseArt(void)
{
    marioAlive.DestroyImage();
    marioDead.DestroyImage();
    platform.DestroyImage();   
    bowserStill.DestroyImage();
    bowserFiring.DestroyImage();
    fireball.DestroyImage();
    lava.DestroyImage();
    bridge.DestroyImage();
    fireballsLeft.DestroyImage();
    fireSound.DestroySound();
    sizzleSound.DestroySound(); 
    MINIGAME::ReleaseArt();
}

bool KILLMARIO::PlayGame(int Difficulty)
{
     
     DisplayGoal("KILL MARIO!");
     
     gameTimer.SetValue(1000);
     difficultySpeed = ((double)(Difficulty) / 0.25);
     gameTimer.SetUPS(-200 - difficultySpeed);
     gameTimer.AddToProcessingList();
     mariosPlatform.topValue = 100;
     mariosPlatform.lowerValue = 400;
     mariosPlatform.speed = 250;
     mariosPlatform.y.SetValue(mariosPlatform.topValue);
     mariosPlatform.y.SetUPS(-1 * (mariosPlatform.speed + difficultySpeed));
     mariosPlatform.y.AddToProcessingList();
     fire[0].onscreen = false;
     fire[1].onscreen = false;
     fire[2].onscreen = false;
     if(Difficulty >= 15)
     {
          fire[0].fired = true;
     }
     else
     {
         fire[0].fired = false;
     }
     if(Difficulty >= 30)
     {
          fire[1].fired = true;
     }
     else
     {
         fire[1].fired = false;
     }
     fire[2].fired = false;
     mario.alive = true;
     
     //Art should already be loaded.
     while(!ButtonEscapeUp() && gameTimer.GetValueFloor() > 0)
     {
           DrawBackgroud();
           DrawFireballsLeft();
           DrawPlatform();
           DrawBowser();
           DrawMario();
           DrawFireball();
           DisplayTimer();
           FlipScreen();           
     }
     
     mariosPlatform.y.RemoveFromProcessingList();
     fire[0].x.RemoveFromProcessingList();
     fire[1].x.RemoveFromProcessingList();
     fire[2].x.RemoveFromProcessingList();
     mario.speed.RemoveFromProcessingList();
     return !mario.alive;
}
