#include <allegro.h>
#include "sprite.h"
#include "graphics.h"

bool InBetween(int LowerValue, int UpperValue, int check);
bool ABoxWithinBBox(int aleft, int aright, int atop, int abottom, int bleft, int bright, int btop, int bbottom);

SPRITE::SPRITE(void)
{
 data = NULL;
}

SPRITE::~SPRITE(void)
{
 this->DestroyImage();
}

void SPRITE::LoadImage(char *filename)
{
     data = load_bitmap(filename, NULL);
}

void SPRITE::DisplayXY(int x, int y)
{
     if(data)
     blit(data, back_buffer, 0, 0, x, y, data->w, data->h);
}

void SPRITE::DisplayMaskedXY(int x, int y)
{
     masked_blit(data, back_buffer, 0, 0, x, y, data->w, data->h);
}

void SPRITE::DisplayRotatedXY(int x, int y, double angle)
{
     rotate_sprite(back_buffer, data, x, y, ftofix(angle));
}

void SPRITE::DestroyImage(void)
{
     if(data)
     {
      destroy_bitmap(data);
      data = NULL;
     }
}

int SPRITE::GetWidth(void)
{
    if(data)
    return data->w;
    else
    return 0;
}

int SPRITE::GetHeight(void)
{
    if(data)
    return data->h;
    else
    return 0;
}

bool InBetween(int LowerValue, int UpperValue, int check)
{
     if(check >= LowerValue && check <= UpperValue)
        return true;
     
     return false;
}

bool ABoxWithinBBox(int aleft, int aright, int atop, int abottom, int bleft, int bright, int btop, int bbottom)
{
     //Check to see if any points are inside the box.
     //Checking Upper Left Corner of A within b (aleft, aup)
     if(InBetween(btop, bbottom, atop) && InBetween(bleft, bright, aleft))
     {
         return true;
     }
     
     //Checking Upper Right Corner
     if(InBetween(btop, bbottom, atop) && InBetween(bleft, bright, aright))
     {
         return true;
     }
     
     //Lower Left
     if(InBetween(btop, bbottom, abottom) && InBetween(bleft, bright, aleft))
     {
         return true;
     }
     
     //Lower right
     if(InBetween(btop, bbottom, abottom) && InBetween(bleft, bright, aright))
     {
         return true;
     }
     
     // if A is like - and B is like | check for +.
     if(atop > btop && abottom < bbottom && aleft < bleft && aright > bright)
     {
         return true;
     }
     
     return false;
}

bool Collision(SPRITE *A, int ax, int ay, SPRITE *B, int bx, int by)
{
     int aleft = ax;
     int aright = ax + A->GetWidth();
     int atop = ay;
     int abottom = ay + A->GetHeight();
     
     int bleft = bx;
     int bright = bx + B->GetWidth();
     int btop = by;
     int bbottom = by + B->GetHeight();
     
     //Check to see if a corner is within another box.  
     return ABoxWithinBBox(aleft, aright, atop, abottom, bleft, bright, btop, bbottom) || ABoxWithinBBox(bleft, bright, btop, bbottom, aleft, aright, atop, abottom);
}
