#ifndef _TROLLRUN_H
#define _TROLLRUN_H

#include "vector.h"
#include "sprite.h"
#include "sound.h"
#include "alfont/alfont.h"

class TROLLRUN
{
private:
        class TROLL
        {
        private:
            SPRITE runA;
            SPRITE runB;
            SPRITE attackA;
            SPRITE attackB;
            VECTOR animRun;  //Time for each frame of running between a & b.
            bool attacking;
        public:
            TROLL();
            ~TROLL();    
            void LoadArt(void); //Loads the art.  Sets stuff up for use.
            void SetAttack(void); //Tells it to club.  (club is only held down for a certain time limit though.  AKA A button down.
            void SetUnattack(void); //This tells it to unclub.  AKA A button up.
            //Displays on the screen at location X, using the sine algorithm.
            void Display(int screenX, int sineX, double angle);
            void Animate(bool animate = true);  //Tells it to animate or not (usefull for pasuing).
        };

        class PEASANT
        {
        private:
                SPRITE *runA;
                SPRITE *runB;
                SPRITE *deadSprite;
                VECTOR xpos;              //Relative to distanceX
                VECTOR aniset;
                bool InGame;
                bool killable;
                bool dead;
                bool escaped;
        public:
               PEASANT()
               {
                   dead = false;
                   InGame = false;
                   escaped = false;
               }
               ~PEASANT()
               {
               }
               
               bool IsAlive(void) { return !dead; }
               
               void LinkToSpriteArt(SPRITE *runAnimationA, SPRITE *runAnimationB, SPRITE *deadAnimation)
               {
                    runA = runAnimationA;
                    runB = runAnimationB;
                    deadSprite = deadAnimation;
               }
               
               void SpawnPeasant(int xDistance, double speed, bool killable)
               {
                    escaped = false;
                    InGame = true;
                    dead = false;
                    xpos.SetValue(xDistance);
                    xpos.SetUPS(speed);
                    xpos.AddToProcessingList();
                    
                    aniset.SetValue(0);
                    aniset.SetUPS(10);
                    aniset.SetMax(2);
                    aniset.SetMin(-10);
                    aniset.SetResetValue(0);
                    aniset.EnableResettingAtBoundry(true);
                    aniset.AddToProcessingList();
               }
               
               void Animate(bool animate = true)  //Good for pausing.
               {
                    if(animate)
                    {
                        xpos.AddToProcessingList();
                        aniset.AddToProcessingList();
                    }
                    else
                    {
                        xpos.RemoveFromProcessingList();
                        aniset.RemoveFromProcessingList();
                    }
               }
               
               //If 
               bool GetEscaped(void)
               {
                    return escaped;
               }
               
               void SetEscaped()
               {
                    escaped = true;
               }
               
               void DestroyPeasant(void)
               {
                    dead = true;
                    escaped = true;
                    if(InGame)
                    {
                     InGame = false;
                     aniset.RemoveFromProcessingList();
                     xpos.RemoveFromProcessingList();
                    }
               }
               
               void Draw(int X, int Y, double angle)
               {
                    if(InGame && aniset.GetValueFloor() < 1)
                    {
                        runA->DisplayRotatedXY(X, Y, angle);
                    }
                    else if (InGame)
                    {
                        runB->DisplayRotatedXY(X,Y,angle);
                    }
                    
                    if(dead)
                    {
                        deadSprite->DisplayRotatedXY(X,Y,angle);
                    }
               }
               
               int GetXLoc(void) { return xpos.GetValueFloor(); }
        };
        
        #define exploderParticles 40
        class EXPLODER
        {
        public:
            EXPLODER()
            {
                yUPSDecrease = 0;
                next = NULL;
                particle = NULL;
            }
            ~EXPLODER() { destroy_bitmap(particle); }
            BITMAP *particle;
            EXPLODER *next;
            VECTOR transparancy;          //When transparancy goes to 0, we delete the exploder.
            VECTOR x[exploderParticles];
            VECTOR y[exploderParticles];
            double yUPSDecrease;
            void DeleteAllSubNodes(void)
            {
                 if(next != NULL)
                 {
                         next->DeleteAllSubNodes();
                 }
                 if(next != NULL)
                 {
                     delete next;
                     next = NULL;
                 }
                 return;
            }
        };
        
        class EXPLODERLIST
        {
        private:
                EXPLODER *head;
        public:
               EXPLODERLIST() { head = NULL; }
               ~EXPLODERLIST() { ClearAll(); }
               void AddExploder(int startingXoffset, int startingY);
               void DrawAndUpdateAll(int xOffset); //Draws the exploders, also deletes exploders that have reached transparancy 0.
               void ClearAll(void);
        };
        
        class HITMISSBADBANNER
        {
        private:
                ALFONT_FONT *font;
                VECTOR translucency;
                int output;  //Hit = 0 Miss = 1 BAD = 2
        public:
               HITMISSBADBANNER(void) 
               { 
                   font = NULL;
               }
               void LoadFont(void)
               {
                   font = alfont_load_font("carbon.ttf");
                   alfont_set_font_size(font, 24);
               }
               void DestroyFont()
               {
                    alfont_destroy_font(font);
               }
               void NewText(int type) { 
                    output = type;
                    translucency.SetUPS(-700);
                    translucency.SetValue(255);
                    translucency.SetMin(0);
                    translucency.SetMax(700);
                    translucency.EnableStoppingAtBoundry(true);
                    translucency.AddToProcessingList();
                    }  //Hit = 0 Miss = 1 BAD = 2
               void DrawBannerCenter(int x, int y)
               {
                    y=y - ((255 / 10) - translucency.GetValueFloor() / 10);
                    if(translucency.GetValueFloor() > 0)
                    {
                        BITMAP *temp = create_bitmap(640, 24);
                        clear_to_color(temp, makecol(255, 0, 255));
                        if(output == 0)
                            alfont_textout_centre_ex(temp, font, "HIT", x, 0, makecol(220, 0, 50), makecol(255, 0, 255));
                        if(output == 1)
                            alfont_textout_centre_ex(temp, font, "MISS", x, 0, makecol(50, 220, 50), makecol(255, 0, 255));
                        if(output == 2)
                            alfont_textout_centre_ex(temp, font, "BAD", x, 0, makecol(50, 0, 220), makecol(255, 0, 255));
                        set_trans_blender(255, 255, 255, translucency.GetValueFloor());
                        draw_trans_sprite(back_buffer, temp, 0, y);
                        destroy_bitmap(temp);
                    }
               }
        }; 
        
        BITMAP *sinehills;
        BITMAP *sky;
        SPRITE hillbg;
        SPRITE cloud;
        SPRITE peasantA;
        SPRITE peasantB;
        SPRITE peasantC;
        SPRITE healhBar;
        SPRITE barRed;
        SPRITE barGreen;
        SPRITE miniGameFlag;
        
        SOUNDEFFECT peasantSmush;
        ALFONT_FONT *comboFont;
        ALFONT_FONT *comboFontLarge;
        ALFONT_FONT *percentFont;
        
        #define numPeasants 10     //Make sure these 2 match up.
        PEASANT peasants[numPeasants];
        EXPLODERLIST exploders;
        HITMISSBADBANNER hitMissBanner;
        TROLL troll;
        bool playing;
        
        VECTOR distanceX;             //How fast the troll runs (updates distanceX). 
        
        int cloudAY;
        int cloudBY;
        int cloudCY;
        int cloudDY;
        
        VECTOR comboTrans;
        int combo;
        int misses;
        int hits;
        int bads;
        int health;
        int nextMiniGamePoint;
        int score;
        int scoreToUnlockMario;
        int scoreToUnlockDisk;
        
        VECTOR cloudAX;
        VECTOR cloudBX;
        VECTOR cloudCX;
        VECTOR cloudDX;
        
        int GenerateCloudY(void);
        
        void DrawSineHillsInMemory(void); //Generates the hill bitmap.
        void DrawSkyInMemory(int redtop, int greentop, int bluetop, int redbottom, int greenbottom, int bluebottom);
        int CalculateYFromSineHill(int x);
        double CalculateTangentAngleForX(int x);
        void DrawSky(void);
        void LoadSprites(void);
        void DestroySprites(void);
        void InitPeasants(void);
        void DrawHillBG();
        void DrawSineHill(unsigned int xoffset);
       void DrawBG(void);
       void DrawClouds(void);
       void SetPeasantLoc(int peasantNumber, int Loction, double speed);
       void DrawAllPeasants(void);
       void CollisionDetectPeasants(void);  //If peasants are within a certain x range, they asplode.
       void DrawHealthBar(void);
       //void AddExploder(int offsetX);
       void DrawComboText(void);
       void InitMisc(void);
       void PauseAction(void);
       void ResumeAction(void);
       void DisplayMiniGameFlags(void);  //Calculates if there's a minigame flag on screen, then draws it.
       void DisplayScore(void);
       void ScrollMiniGameAcrossScreen(void);
       void ShowWinnerLoser(bool result);
       void DisplayIntro(void);
       void DisplayGameOver(void);       
       void DisplayResults(void);
public:
       TROLLRUN();
       void LoadArt(void);
       void ReleaseArt(void);
       void DoTrollRun(void);
};

#endif
