#include <allegro.h>
#include "timer.h"
#include "vector.h"

//Since all the vectors are defined outside of the Vector list, I just need to clean up the VECTOR_NODEs.
class VECTOR_LIST
{
private:
        class VECTOR_NODE
        {
         public:
         VECTOR_NODE();
         ~VECTOR_NODE();
         VECTOR_NODE *next;                    //Points to the next vector in the list.
         VECTOR *cur;                    //Previous Node.
        };        
        VECTOR_NODE *head;
        
        void RemoveTail(void);                  //Removes the tail.  
public:
       VECTOR_LIST();
       ~VECTOR_LIST();
       void AddVector(VECTOR *addMe);
       void RemoveVector(VECTOR *removeMe);
       void UpdateAll(void);
};

VECTOR_LIST AllVectors;

void UpdateAllVectors(void)
{
     AllVectors.UpdateAll();
}

VECTOR::VECTOR(void)
{
                    inProcessList = false;
                    mainValue = 0;
                    updateValue = 1.0;
                    minValue = 0;
                    maxValue = 0;
                    resetOnBoundry = false;
                    resetValue = 0;
                    stopOnBoundry = false;
}

VECTOR::~VECTOR(void)
{
    if(inProcessList)
         RemoveFromProcessingList();                
}

void VECTOR::AddToProcessingList(void)
{
     if(!inProcessList)
     AllVectors.AddVector(this);
     inProcessList = true;
}

void VECTOR::RemoveFromProcessingList(void)
{
     if(inProcessList)
     AllVectors.RemoveVector(this);
     inProcessList = false;
}

void VECTOR::Update(void)
{
     double frameTime = FrameTime();
     mainValue = mainValue + (updateValue * frameTime);
     
     if(resetOnBoundry)
     {
        if(mainValue >= maxValue)
        {
           mainValue = (mainValue - maxValue) + resetValue;
        }
        else if(mainValue <= minValue)
        {
             mainValue = (mainValue - minValue) + resetValue;
        }
     }
     else if(stopOnBoundry)
     {
         if(mainValue >= maxValue)
        {
           //Don't remove stuff from the processing list whilst it is being processed.
           SetUPS(0);
        }
        else if(mainValue <= minValue)
        {
             SetUPS(0);
        }
     }
}

double VECTOR::GetValue(void)
{
       return mainValue;
}

int VECTOR::GetValueFloor(void)
{
    return (int)(mainValue);
}

void VECTOR::SetValue(double newValue)
{
     mainValue = newValue;
}

void VECTOR::SetValue(int newValue)
{
     mainValue = (double)(newValue);
}


void VECTOR::SetUPS(double updatesPerSecond)
{
     updateValue = updatesPerSecond;
}
       
void VECTOR::SetMax(double max)
{
     maxValue = max;
}

void VECTOR::SetMin(double min)
{
     minValue = min;
}

void VECTOR::EnableResettingAtBoundry(bool reset)
{
     if(resetOnBoundry = reset)
     {
                stopOnBoundry = false;   
     }
}

void VECTOR::SetResetValue(double ResetValue)
{
     resetValue = ResetValue;
}

void VECTOR::EnableStoppingAtBoundry(bool stop)
{
     if(stopOnBoundry = stop)
     {
                 resetOnBoundry = false;     
     }
}

VECTOR_LIST::VECTOR_NODE::VECTOR_NODE()
{
    cur = NULL;
    next = NULL;
}

VECTOR_LIST::VECTOR_NODE::~VECTOR_NODE()
{
}

VECTOR_LIST::VECTOR_LIST()
{
    head = NULL;
}

VECTOR_LIST::~VECTOR_LIST()
{
    while(head != NULL)
        RemoveTail();
}

void VECTOR_LIST::AddVector(VECTOR *addMe)
{
     VECTOR_NODE *newNode;
     VECTOR_NODE *runner;
     
     newNode = new VECTOR_NODE;
     newNode->cur = addMe;
     
     if(head == NULL)
     {
         head = newNode;
     }
     else
     {
         runner = head;
         while(runner->next != NULL)
         {
             if(runner->cur == addMe)
                 return;
             runner = runner->next;
         }
         newNode = new VECTOR_NODE;
         newNode->cur = addMe;
         runner->next = newNode;
     }
}

void VECTOR_LIST::RemoveVector(VECTOR *removeMe)
{
     VECTOR_NODE *runner = head;
     VECTOR_NODE *deleteme;
     
     if(runner != NULL)
     {
        if(runner->cur == removeMe)
        {
              deleteme = head;
              head = head->next;
              delete deleteme;
              return;
        }       
        else
        {
            while(runner->next != NULL && runner->next->cur != removeMe)
            {
                runner = runner->next;
            }
            if(runner->next == NULL)
                return;
            else
            {
                deleteme = runner->next;
                runner->next = deleteme->next;
            }
        }
        delete deleteme;
     }
}

void VECTOR_LIST::RemoveTail(void)
{
     VECTOR_NODE *deleteMe;
     VECTOR_NODE *runner;
     
     runner = head;
     if(runner->next == NULL)
     {
         deleteMe = runner;
         head = NULL;
         delete deleteMe;
     }
     else
     {
         while(runner->next->next != NULL)
         {
             runner = runner->next;
         }
         deleteMe = runner->next;
         runner->next = NULL;
         delete deleteMe;
     }
}

void VECTOR_LIST::UpdateAll(void)
{
     VECTOR_NODE *runner = head;
     
     while(runner != NULL)
     {
         runner->cur->Update();
         runner = runner->next;
     }
}
